TOKEN_QUERY = """
        query GetTokens($address: String) {
            Token(
                where: { address: { _eq: $address } },
                order_by: { tokenId: asc }
            ) {
                tokenId
                address
                minter
                owner
                metadata {
                json
                }
            }
        }
        """

TOKEN_TRANSFERS_QUERY = """
        query GetTransferTokens($address: String) {
            Token(
                where: { address: { _eq: $address } },
                order_by: { tokenId: asc }
            ) {
                transferEvents { id }
            }
        }
        """

CONTRACT_QUERY = """
query GetContract($address: String) {
        TokenContract(
            where: { address: { _eq: $address } }
        ) {
            address
            name
            supportsMetadata
            symbol
        }
    }
"""

LIST_NFT_QUERY = """
    {
        TokenContract {
        address
        name
        supportsMetadata
        symbol
        }
    }
"""

METADATA_QUERY = """
query GetMetadata($address: String) {
        TokenMetadata(
            where: { address: { _eq: $address } }
        ) {
    address # Token address
    id # uuid, ignore
    json # data blob
    tokenId
    tokenURI
  }
}
"""

TRANSACTION_QUERY = """
query GetTransferTokens($hash: String) {
            Transaction(
                where: { hash: { _eq: $hash } }
    ) {
    # Fields
    blockHash
    blockNumber
    blockTimestamp
    failureReason
    from
    gas
    gasPrice
    hash
    input
    network
    nonce
    status
    to
    transactionIndex
    value

    # Relationships
    eventLogs {
      id
    }
    mediaMints {
      id
    } # see MediaMint table for context
  }
}
"""



from zora.queries import LIST_NFT_QUERY
from zora.utils import run_zora_query
from zora.nft import NFT


def list_nft():

    data = run_zora_query(LIST_NFT_QUERY)
    result = []
    for contract in data["data"]["TokenContract"]:
        result.append(NFT(contract["address"]))
    return result



import asyncio
import aiohttp

from zora.utils import run_zora_query
from zora.queries import (
    TOKEN_QUERY,
    CONTRACT_QUERY,
    METADATA_QUERY,
    TOKEN_TRANSFERS_QUERY,
    TRANSACTION_QUERY,
)
from zora.base import BaseToken


class NFT(BaseToken):
    def __init__(self, address) -> None:

        response = self._get_contract(address)
        response = response["data"]["TokenContract"][0]
        for k, v in response.items():
            setattr(self, k, v)

    def _get_contract(self, address):
        variables = {"address": address}
        return run_zora_query(CONTRACT_QUERY, variables)

    def get_items(self):
        variables = {"address": self.address}
        response = run_zora_query(TOKEN_QUERY, variables)
        result = []
        for token in response["data"]["Token"]:
            result.append(Token(token))
        return result

    def get_metadata(self):
        variables = {"address": self.address}
        response = run_zora_query(METADATA_QUERY, variables)
        result = []
        for token in response["data"]["TokenMetadata"]:
            result.append(Meatadata(token))
        return result

    def get_transfers(self):

        variables = {"address": self.address}
        response = run_zora_query(TOKEN_TRANSFERS_QUERY, variables)

        result = []
        for tx in response["data"]["Token"]:
            for tx_id in tx["transferEvents"]:
                result.append(tx_id["id"].split("-")[0])
        return result

    async def _get_transaction(self, transactions):
        async def get_tx(session, query, variables=None):

            body = {"query": query}
            if variables is not None:
                body["variables"] = variables

            async with session.post(
                "https://indexer-prod-mainnet.ZORA.co/v1/graphql",
                json=body,
                headers={"X-Hasura-Role": "anonymous"},
            ) as resp:
                response = await resp.json()
                return response

        volume = 0

        async with aiohttp.ClientSession() as session:
            tasks = []
            for tx in transactions:
                variables = {"hash": tx}
                tasks.append(
                    asyncio.ensure_future(get_tx(session, TRANSACTION_QUERY, variables))
                )

            processed_list = await asyncio.gather(*tasks)
            for response in processed_list:
                response = response["data"]["Transaction"][0]
                volume += float(response["value"]) / (10 ** 18)

        return volume

    def get_volume(self, transactions=None):

        volume = 0

        if not transactions:
            transactions = self.get_transfers()

        volume = asyncio.run(self._get_transaction(transactions))
        volume = float("{:.4f}".format(volume))
        return volume


class Meatadata(BaseToken):
    def __init__(self, token):
        for k, v in token.items():
            setattr(self, k, v)


class Token(BaseToken):
    def __init__(self, token):
        for k, v in token.items():
            setattr(self, k, v)



import requests


def run_zora_query(query, variables=None):
    endpoint = "https://indexer-prod-mainnet.ZORA.co/v1/graphql"

    body = {"query": query}

    if variables is not None:
        body["variables"] = variables

    response = requests.post(
        endpoint, json=body, headers={"X-Hasura-Role": "anonymous"}
    )
    return response.json()
